import type { BaseContext, MiddlewareFunction } from './middleware.js';
export type ComposerProperties<MiddlewareFunctions extends unknown[], InputContext extends BaseContext, OutputContext extends BaseContext> = {
    functions: Readonly<MiddlewareFunctions>;
    and: <Context extends BaseContext>(middleware: MiddlewareFunction<OutputContext, Context>) => Composer<[
        ...MiddlewareFunctions,
        MiddlewareFunction<OutputContext, Context>
    ], InputContext, Context>;
};
export type Composer<MiddlewareFunctions extends unknown[], InputContext extends BaseContext, OutputContext extends BaseContext> = MiddlewareFunction<InputContext, OutputContext> & ComposerProperties<MiddlewareFunctions, InputContext, OutputContext>;
export declare class ComposeError extends Error {
    type: 'next_call_twice';
    constructor(type: 'next_call_twice');
}
/**
 * INTERNAL. Should not use it directly since it don't check any type for the initial functions,
 * due to the limit of TypeScript or my coding skill.
 *
 * @param functions An array of (supposed to be) middleware functions.
 * @returns A composer with the given functions.
 */
export declare const createComposer: <T extends unknown[], InputContext extends BaseContext, OutputContext extends BaseContext>(functions: Readonly<T>) => Composer<T, InputContext, OutputContext>;
type EmptyArray = [];
/**
 * Create a chainable composer with the given middleware function.
 * Call `.and()` of the composer to chain another middleware function.
 *
 * Usage:
 * ```ts
 * compose(fn1).and(fn2).and(fn3)
 * ```
 *
 * Each composer itself is a middleware function object,
 * i.e. it has the same call signature as a normal middleware function.
 *
 * @param middleware The first middleware function to compose.
 * @returns A composer.
 */
declare function compose<InputContext extends BaseContext, OutputContext extends BaseContext>(middleware: MiddlewareFunction<InputContext, OutputContext>): Composer<[MiddlewareFunction<InputContext, OutputContext>], InputContext, OutputContext>;
declare function compose<InputContext extends BaseContext = BaseContext, OutputContext extends BaseContext = InputContext>(): Composer<EmptyArray, InputContext, OutputContext>;
export default compose;
