export class ComposeError extends Error {
    constructor(type) {
        super('The sample next() function is going to call twice.');
        this.type = type;
    }
}
const buildNext = ([first, ...rest], next, http) => {
    if (!first) {
        return next;
    }
    // Business need
    // eslint-disable-next-line @silverhand/fp/no-let
    let called = false;
    return async (context) => {
        if (called) {
            throw new ComposeError('next_call_twice');
        }
        // eslint-disable-next-line @silverhand/fp/no-mutation
        called = true;
        return first(context, buildNext(rest, next, http), http);
    };
};
/**
 * INTERNAL. Should not use it directly since it don't check any type for the initial functions,
 * due to the limit of TypeScript or my coding skill.
 *
 * @param functions An array of (supposed to be) middleware functions.
 * @returns A composer with the given functions.
 */
export const createComposer = function (functions) {
    /**
     * TypeScript won't derive the same type after spreading an array.
     * It'll be great to figure out the root cause, but not worthy to be a blocker.
     */
    // eslint-disable-next-line no-restricted-syntax
    const _functions = Object.freeze([...functions]);
    const composer = Object.assign(async function (context, next, http) {
        /**
         * `buildNext()` doesn't care about the context type,
         * and it's also hard to derive the strict context with a generic array type.
         * Thus it's OK to use `as` I think.
         */
        /* eslint-disable no-restricted-syntax */
        return buildNext(_functions, next, http)(context);
        /* eslint-enable no-restricted-syntax */
    }, {
        get functions() {
            return _functions;
        },
        and(middleware) {
            return createComposer(Object.freeze([..._functions, middleware]));
        },
    });
    return composer;
};
function compose(middleware) {
    if (middleware) {
        return createComposer([middleware]);
    }
    return createComposer([]);
}
export default compose;
