/// <reference types="node" resolution-mode="require"/>
import type { IncomingHttpHeaders } from 'node:http';
import type { RequestMethod } from '@withtyped/shared';
import type { BaseContext, HttpContext, NextFunction } from '../middleware.js';
export type WithRequestContext<InputContext> = InputContext & {
    request: {
        /** The unique identifier for the request. An alias of `context.requestId`. */
        id?: string;
        /** The request method. */
        method?: RequestMethod;
        /** The request headers. */
        headers: IncomingHttpHeaders;
        /** The full request URL. */
        url: URL;
        body?: unknown;
    };
};
export type RequestContext = WithRequestContext<BaseContext>;
export type MergeRequestContext<InputContext extends RequestContext, MergeType> = Omit<InputContext, 'request'> & {
    request: InputContext['request'] & MergeType;
};
export default function withRequest<InputContext extends BaseContext>(): (context: InputContext, next: NextFunction<WithRequestContext<InputContext>>, { request: { method, headers, url, socket } }: HttpContext) => Promise<void>;
