import { TLSSocket } from 'node:tls';
import { requestMethods } from '@withtyped/shared';
export default function withRequest() {
    return async (context, next, { request: { method, headers, url, socket } }) => {
        const protocol = socket instanceof TLSSocket ? 'https' : 'http';
        // Empty host will fail when constructing URL
        // which is expected since host is mandatory in HTTP/1.1
        const host = headers.host ?? '';
        return next({
            ...context,
            request: {
                ...(context.requestId && { id: context.requestId }),
                method: requestMethods.find((value) => value === method?.toUpperCase()),
                headers,
                url: new URL(url ?? '', `${protocol}://${host}`),
            },
        });
    };
}
