import type { JsonObject } from '../types.js';
import type { CamelCase, PrimitiveType, PrimitiveTypeMap, RawParserConfig } from './types.js';
export declare const findType: (raw?: string) => PrimitiveType | undefined;
export declare const parseTableName: (raw: string) => string | undefined;
/**
 * Parse a SQL string to column configs with camelCase keys.
 *
 * @param raw The raw SQL string.
 * @returns An object with camelCase keys, while their values are the column configs read from SQL.
 */
export declare const parseRawConfigs: (raw: string) => Record<string, RawParserConfig>;
export declare const parsePrimitiveType: (value: unknown, type: PrimitiveType) => PrimitiveTypeMap[PrimitiveType] | undefined;
export declare const isObject: (value: unknown) => value is JsonObject;
export declare const camelCase: <T extends string>(value: T) => CamelCase<T>;
/**
 * Convert the top-level keys to camelCase if the value is a plain object.
 * If there are multiple keys with the same camelCase result, the value of that key will NOT be ensured.
 */
export declare const camelCaseKeys: (value: unknown) => unknown;
