import { z } from 'zod';
const typeToGuard = Object.freeze({
    boolean: z.boolean(),
    number: z.number(),
    string: z.string(),
    json: z.record(z.unknown()).or(z.array(z.unknown())),
    date: z.date(),
});
const convertRawConfig = (config) => {
    // eslint-disable-next-line @silverhand/fp/no-let
    let guard = typeToGuard[config.type];
    if (config.isArray) {
        // eslint-disable-next-line @silverhand/fp/no-mutation
        guard = guard.array();
    }
    if (config.isNullable) {
        // eslint-disable-next-line @silverhand/fp/no-mutation
        guard = guard.nullable();
    }
    return guard;
};
export const convertConfigToZod = (config, extendedConfig, forType) => {
    const parser = extendedConfig?.parser ?? convertRawConfig(config);
    switch (forType) {
        case 'model': {
            return parser;
        }
        case 'create': {
            if (extendedConfig?.default) {
                const { default: defaultValue, readonly } = extendedConfig;
                return readonly
                    ? z
                        .undefined()
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                        .transform(() => (typeof defaultValue === 'function' ? defaultValue() : defaultValue))
                    : parser.default(defaultValue);
            }
            if (extendedConfig?.readonly) {
                return z.undefined();
            }
            if (config.hasDefault) {
                return parser.optional();
            }
            return parser;
        }
        case 'patch': {
            if (extendedConfig?.readonly) {
                return z.undefined();
            }
            return parser.optional();
        }
    }
};
