import type { Nullable } from '@silverhand/essentials';
import type { QueryResult } from './types.js';
import type { Sql } from './utils.js';
export declare abstract class Queryable<SqlTag extends Sql> {
    /** Run query and return rows only. */
    any<T extends Record<string, unknown> = Record<string, unknown>>(sql: SqlTag): Promise<T[]>;
    /**
     * Run query and return rows only.
     *
     * @throws NoResultError if no result found.
     */
    many<T extends Record<string, unknown> = Record<string, unknown>>(sql: SqlTag): Promise<T[]>;
    /**
     * Run query and return the first row only or `null`.
     *
     * @throws MultipleRowsFoundError if the result has multiple rows.
     */
    maybeOne<T extends Record<string, unknown> = Record<string, unknown>>(sql: SqlTag): Promise<Nullable<T>>;
    /**
     * Run query and return the first row only.
     *
     * @throws NoResultError if no result found.
     * @throws MultipleRowsFoundError if the result has multiple rows.
     */
    one<T extends Record<string, unknown> = Record<string, unknown>>(sql: SqlTag): Promise<T>;
    /** Run query and return a boolean value that indicates whether the query produces result. */
    exists(sql: SqlTag): Promise<boolean>;
    /** Run arbitrary SQL query, return a standard query result. */
    abstract query<T extends Record<string, unknown> = Record<string, unknown>>(sql: SqlTag): Promise<QueryResult<T>>;
}
export declare abstract class Transaction<SqlTag extends Sql = Sql> extends Queryable<SqlTag> {
    abstract start(): Promise<void>;
    abstract end(): Promise<void>;
}
export default abstract class QueryClient<SqlTag extends Sql = Sql> extends Queryable<SqlTag> {
    abstract connect<T>(config?: T): Promise<void>;
    abstract end(): Promise<void>;
    abstract transaction(): Promise<Transaction<SqlTag>>;
}
