import { MultipleRowsFoundError, NoResultError } from './errors.js';
export class Queryable {
    /** Run query and return rows only. */
    async any(sql) {
        const { rows } = await this.query(sql);
        return rows;
    }
    /**
     * Run query and return rows only.
     *
     * @throws NoResultError if no result found.
     */
    async many(sql) {
        const { rows } = await this.query(sql);
        if (rows.length === 0) {
            throw new NoResultError(sql);
        }
        return rows;
    }
    /**
     * Run query and return the first row only or `null`.
     *
     * @throws MultipleRowsFoundError if the result has multiple rows.
     */
    async maybeOne(sql) {
        const { rows } = await this.query(sql);
        if (rows.length > 1) {
            throw new MultipleRowsFoundError(sql);
        }
        return rows[0] ?? null;
    }
    /**
     * Run query and return the first row only.
     *
     * @throws NoResultError if no result found.
     * @throws MultipleRowsFoundError if the result has multiple rows.
     */
    async one(sql) {
        const { rows } = await this.query(sql);
        if (rows.length > 1) {
            throw new MultipleRowsFoundError(sql);
        }
        if (!rows[0]) {
            throw new NoResultError(sql);
        }
        return rows[0];
    }
    /** Run query and return a boolean value that indicates whether the query produces result. */
    async exists(sql) {
        const { rows } = await this.query(sql);
        return rows.length > 0;
    }
}
export class Transaction extends Queryable {
}
export default class QueryClient extends Queryable {
}
