import { pipeline } from 'node:stream/promises';
import { promisify } from 'node:util';
import { contentTypes } from '@withtyped/shared';
export const getWriteResponse = (response) => promisify((chunk, callback) => {
    if (response.writableEnded || response.destroyed) {
        callback(new Error("Unable to write response since it's already ended or destroyed."));
        return;
    }
    if (chunk instanceof Buffer || chunk instanceof Uint8Array) {
        return response.write(chunk, callback);
    }
    if (typeof chunk === 'string') {
        return response.write(chunk, 'utf8', callback);
    }
    response.write(JSON.stringify(chunk), 'utf8', callback);
});
export const writeContextToResponse = async (response, { status, headers, json, stream }) => {
    // Send status along with headers
    if (status !== 'ignore') {
        // eslint-disable-next-line @silverhand/fp/no-mutation
        response.statusCode = status ?? 404;
    }
    if (json) {
        response.setHeader('Content-Type', contentTypes.json);
    }
    if (headers) {
        for (const [key, value] of Object.entries(headers)) {
            if (value) {
                response.setHeader(key, value);
            }
        }
    }
    // Send JSON body
    if (json) {
        await getWriteResponse(response)(json);
    }
    // Pipe stream
    if (stream) {
        await pipeline(stream, response);
    }
};
