import type { RequestMethod } from '@withtyped/shared';
import type { RequestContext } from '../middleware/with-request.js';
import type { MiddlewareFunction } from '../middleware.js';
import type { OpenAPIV3 } from '../openapi/openapi-types.js';
import type { Parser } from '../types.js';
import type { RouteLike } from './route/index.js';
import { type BuildRoute } from './types.build-route.js';
import type { BaseRoutes, MergeRoutes, NormalizedPrefix, RoutesWithPrefix } from './types.js';
export * from './types.js';
export declare const openApiRoute = "/openapi.json";
export type MethodRoutesMap<InputContext extends RequestContext> = Record<string, Array<RouteLike<InputContext>>>;
type BaseRouter<PreInputContext extends RequestContext, InputContext extends RequestContext, Routes extends BaseRoutes, Prefix extends string> = {
    [key in Lowercase<RequestMethod>]: BuildRoute<PreInputContext, InputContext, Routes, Prefix, key>;
};
export type RouterRoutes<RouterInstance> = RouterInstance extends Router<infer _, infer __, // Use different names to avoid conflict since currently we cannot ignore inference. See https://github.com/microsoft/TypeScript/issues/26242
infer Routes> ? Routes : never;
export default class Router<PreInputContext extends RequestContext = RequestContext, InputContext extends RequestContext = RequestContext, Routes extends BaseRoutes = BaseRoutes, Prefix extends string = ''> implements BaseRouter<PreInputContext, InputContext, Routes, Prefix> {
    get: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "get">;
    post: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "post">;
    put: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "put">;
    patch: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "patch">;
    delete: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "delete">;
    copy: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "copy">;
    head: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "head">;
    options: BuildRoute<PreInputContext, InputContext, Routes, Prefix, "options">;
    readonly prefix: string;
    protected readonly routesMap: MethodRoutesMap<InputContext>;
    protected readonly middlewareArray: Array<MiddlewareFunction<PreInputContext, InputContext>>;
    /**
     * Create a router instance.
     */
    constructor();
    /**
     * Create a router instance with a normalized prefix:
     * - Start with `/`, but not end with `/`
     * - Have NO continuous `/`, e.g. `/foo//bar`
     * - Have NO path parameter, e.g. `/:foo`
     */
    constructor(prefix: NormalizedPrefix<Prefix>);
    routes(): MiddlewareFunction<PreInputContext, InputContext | PreInputContext>;
    use<NewInputContext extends InputContext>(middleware: MiddlewareFunction<InputContext, NewInputContext>): Router<PreInputContext, NewInputContext, Routes, Prefix>;
    withOpenApi(parseSearch?: <T>(guard?: Parser<T>) => OpenAPIV3.ParameterObject[], parse?: <T>(guard?: Parser<T>) => OpenAPIV3.SchemaObject, info?: Partial<OpenAPIV3.InfoObject>): import("./types.build-route.js").RouterWithRoute<PreInputContext, InputContext, Routes, Prefix, "get", "/openapi.json", unknown, unknown, OpenAPIV3.Document<{}>>;
    pack<AnotherRoutes extends BaseRoutes, AnotherInputContext extends InputContext extends AnotherInputContext ? RequestContext : never>(another: Router<AnotherInputContext, AnotherInputContext, AnotherRoutes, string>): Router<PreInputContext, InputContext, MergeRoutes<Routes, RoutesWithPrefix<AnotherRoutes, Prefix>>, Prefix>;
    findRoute<Method extends Lowercase<RequestMethod>>(method: Method, path: keyof Routes[Method]): RouteLike<InputContext, string, unknown, unknown, unknown> | undefined;
    private buildRoute;
}
export type CreateRouter = {
    <InputContext extends RequestContext>(): Router<InputContext, InputContext>;
    <InputContext extends RequestContext, Prefix extends string>(prefix: NormalizedPrefix<Prefix>): Router<InputContext, InputContext, BaseRoutes, NormalizedPrefix<Prefix>>;
};
export declare const createRouter: CreateRouter;
export type { RouterWithRoute, BuildRoute } from './types.build-route.js';
