import { describe, it } from 'node:test';
import { expectTypeOf } from 'expect-type';
import { z } from 'zod';
import Router from './index.js';
const mid1 = async (context, next) => {
    return next({ ...context, foo: 'bar' });
};
void describe('router types', () => {
    void it('should be able to infer types for a complex router', () => {
        const subRouter = new Router().get('/bar', { response: z.object({ storeIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { storeIds: [] } });
        });
        const router = new Router()
            .use(mid1)
            .get('/foo', { response: z.object({ bookIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { bookIds: [] } });
        })
            .pack(subRouter);
        // Type tests, should report compile errors if the types are not inferred correctly
        expectTypeOf().toEqualTypeOf();
        expectTypeOf().toEqualTypeOf();
    });
    void it('should be able to `.pack()` another router with "lower" context types', () => {
        const subRouter = new Router().get('/bar', { response: z.object({ storeIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { storeIds: [] } });
        });
        const router = new Router()
            .use(mid1)
            .get('/foo', { response: z.object({ bookIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { bookIds: [] } });
        })
            .pack(subRouter);
        // Type tests, should report compile errors if the types are not inferred correctly
        expectTypeOf().toEqualTypeOf();
        expectTypeOf().toEqualTypeOf();
    });
    void it('should forbid to `.pack()` another router with incompatible context types', () => {
        const subRouter = new Router().get('/bar', { response: z.object({ storeIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { storeIds: [] } });
        });
        const router = new Router()
            .use(mid1)
            .get('/foo', { response: z.object({ bookIds: z.string().array() }) }, async (context, next) => {
            return next({ ...context, json: { bookIds: [] } });
        });
        // @ts-expect-error
        router.pack(subRouter);
    });
});
