import { contentTypes } from '@withtyped/shared';
const defaultInfo = { title: 'API reference', version: '0.1.0' };
const defaultResponses = {
    '204': {
        description: 'No content',
    },
};
const parseParameters = (path) => path
    .split('/')
    .filter((part) => part.startsWith(':'))
    .map((part) => ({
    name: part.slice(1),
    in: 'path',
    required: true,
    schema: { type: 'string' },
}));
export const buildOpenApiJson = (routesMap, parseSearch, parse, info) => {
    const tryParse = (guard) => {
        return parse?.(guard);
    };
    const pathMap = new Map();
    for (const [method, routes] of Object.entries(routesMap)) {
        for (const { fullPath, path, guard } of routes) {
            const operationObject = {
                parameters: [...parseParameters(path), ...(parseSearch?.(guard.search) ?? [])],
                requestBody: guard.body && {
                    required: true,
                    content: { [contentTypes.json]: { schema: tryParse(guard.body) } },
                },
                responses: guard.response
                    ? {
                        '200': {
                            description: 'OK',
                            content: { [contentTypes.json]: { schema: tryParse(guard.response) } },
                        },
                    }
                    : defaultResponses,
            };
            pathMap.set(fullPath, {
                ...pathMap.get(fullPath),
                [method]: operationObject,
            });
        }
    }
    return {
        openapi: '3.0.1',
        info: { ...defaultInfo, ...info },
        paths: Object.fromEntries(pathMap),
    };
};
