import type { RequestContext } from '../../middleware/with-request.js';
import type { HttpContext, MiddlewareFunction, NextFunction } from '../../middleware.js';
import type { GuardedContext, RequestGuard } from '../types.js';
export declare abstract class RouteLike<InputContext extends RequestContext, Path extends string = string, Search = unknown, Body = unknown, JsonResponse = unknown> {
    abstract prefix: string;
    abstract path: Path;
    abstract readonly fullPath: string;
    abstract runnable: MiddlewareFunction<InputContext, InputContext & {
        json?: JsonResponse;
    }>;
    abstract guard: RequestGuard<Search, Body, JsonResponse>;
    abstract clone(newPrefix?: string): RouteLike<InputContext, Path, Search, Body, JsonResponse>;
}
export default class Route<InputContext extends RequestContext, Path extends string = string, Search = unknown, Body = unknown, JsonResponse = unknown> implements RouteLike<InputContext, Path, Search, Body, JsonResponse> {
    readonly prefix: string;
    readonly path: Path;
    readonly guard: RequestGuard<Search, Body, JsonResponse>;
    protected run: MiddlewareFunction<GuardedContext<InputContext, Path, Search, Body>, InputContext & {
        json?: JsonResponse;
    }>;
    constructor(prefix: string, path: Path, guard: RequestGuard<Search, Body, JsonResponse>, run: MiddlewareFunction<GuardedContext<InputContext, Path, Search, Body>, InputContext & {
        json?: JsonResponse;
    }>);
    get fullPath(): `${string}${Path}`;
    get runnable(): (context: InputContext, next: NextFunction<InputContext & {
        json?: JsonResponse;
    }>, http: HttpContext) => Promise<void>;
    clone(newPrefix?: string): Route<InputContext, Path, Search, Body, JsonResponse>;
}
