import RequestError from '../../errors/RequestError.js';
import { guardInput } from './utils.js';
export class RouteLike {
}
export default class Route {
    constructor(prefix, path, guard, run) {
        this.prefix = prefix;
        this.path = path;
        this.guard = guard;
        this.run = run;
    }
    get fullPath() {
        return `${this.prefix}${this.path}`;
    }
    get runnable() {
        return async (context, next, http) => {
            try {
                // We don't want to catch errors during running, this try-catch is for `guardInput()`
                // eslint-disable-next-line @typescript-eslint/return-await
                return this.run({
                    ...context,
                    guarded: guardInput(this.prefix, this.path, context.request.url, context.request.body, this.guard),
                }, next, http);
            }
            catch (error) {
                if (error instanceof RequestError) {
                    throw error;
                }
                throw new RequestError(error instanceof Error ? error.message : 'Request input error', 400, error);
            }
        };
    }
    clone(newPrefix = this.prefix) {
        return new Route(newPrefix, this.path, this.guard, this.run);
    }
}
