import type { Guarded, Params, RequestGuard } from '../types.js';
/**
 * Parse `match`'s path segments start with `:` with the value of the same index in the `url`'s path segments into an object.
 * This function assumes `match` has been normalized.
 *
 * E.g.
 * ```ts
 * parsePathParams('/foo/:bar/:baz', new URL('/foo/123/abc', base)) // { bar: '123', baz: 'abc' }
 * parsePathParams('/foo/:bar/:baz', new URL('/foo/123', base)) // { bar: '123', baz: '' }
 * ```
 */
export declare const parsePathParams: <Path extends string>(match: Path, { pathname }: URL) => Params<Path>;
/**
 * Parse URLSearchParams to an key-value object. If a key appears multiple times, the value will be an array.
 *
 * E.g.
 * ```ts
 * searchParamsToObject(new URLSearchParams('?foo=1&bar=2')) // { foo: '1', bar: '2' }
 * searchParamsToObject(new URLSearchParams('?foo=1&bar=2&foo=%5Ea')) // { foo: ['1', '^a'], bar: '2' }
 * ```
 */
export declare const searchParamsToObject: (urlSearchParams: URLSearchParams) => Record<string, string | string[]>;
export declare const guardInput: <Path extends string, Search, Body_1>(prefix: string, path: Path, url: URL, body: unknown, guard: RequestGuard<Search, Body_1, unknown>) => Guarded<Path, Search, Body_1>;
