import { IncomingMessage, ServerResponse } from 'node:http';
import { Socket } from 'node:net';
import { TLSSocket } from 'node:tls';
import sinon from 'sinon';
export const createHttpContext = (isHttps = false) => {
    const request = new IncomingMessage(isHttps ? new TLSSocket(new Socket()) : new Socket());
    return {
        request,
        response: new ServerResponse(request),
    };
};
export const createRequestPayload = (isHttps = false) => {
    const { request, response } = createHttpContext(isHttps);
    return [request, response];
};
export const createRequestContext = (method, path, body) => Object.freeze({
    request: {
        method,
        url: new URL(path, 'https://localtest'),
        headers: {},
        body,
    },
});
export const stubResponseWrite = (response) => sinon.stub(response, 'write').callsFake((_, callback, callback2) => {
    if (typeof callback === 'function') {
        // @ts-expect-error compatible with the function overload
        callback(null);
    }
    else {
        callback2?.(null);
    }
    return true;
});
